function [x, j, res_norm, res_hist] = mrs3(alpha, S, x, r, tol, maxiter)
%MRS3 Minimal Residual method for Shifted Skew-Symmetric systems
%
%  This function solves the shifted skew-symmetric matrix system A*x = b,
%  where A = alpha*I + S with S skew-symmetric, using the MRS3 algorithm.
%  Author      :  Reijer Idema (http://www.reijeridema.nl)
%  Last edited :  15 September 2008
%
%  Parameters
%     alpha    :  shift of the coefficient matrix
%     S        :  skew-symmetric part of the coefficient matrix
%     x        :  initial solution x_0
%     r        :  initial residual r_0 = b - A*x_0
%     tol      :  stop criterion on the relative residual norm
%     maxiter  :  stop criterion on the number of iterations
%
%  Output
%     x        :  approximate solution to the system A*x = b
%     j        :  iteration count
%     res_norm :  residual norm
%     res_hist :  residual norm history through all iterations
%
% Complexity
%     matrix-vector     :  1
%     innter products   :  1
%     vector updates    :  3
%
% Memory requirement
%     matrix memory     :  1
%     vector memory     :  5 (not counting res_hist)

%  Note that there are multiple pairs of vectors, like q_j and q_{j-1},
%  that need to be shifted in each iteration, i.e.,
%  create q_{j+1}, q_j=q_{j-1}, clear q_{j-1}, which is very slow.
%  For efficiency each of these vector pairs has been implemented as a
%  matrix of the form Q = [ q_A, q_B ].
%  In each iteration the oldest of the two matrix columns is updated,
%  while the other one is conserved.
%  To this purpose a set of indices it and io, that describe which column
%  is which, are kept and updated in each iteration.
%  In programming languages that support pointers, the same effect will be
%  easy to accomplish by working with 2 vectors q_A and q_B,
%  and swapping the pointers to them in each iteration.

%% Initialize variables
j = 0;

norm_r0 = norm(r,2);                            % norm_r0 = ||r_0||
eps = -norm_r0;                                 % eps = v_j(j+1)
res_norm = norm_r0;                             % res_norm = ||r_0||
res_hist = zeros(maxiter+1, 1);                 % residual history preallocation
res_hist(1) = res_norm;

it = 1; io = 2;                                 % index_this & index_other
dim = size(S,1);                                % problem dimension
Q = [ zeros(dim,1) r ];                         % Q = [ q_0 p_1 ]
B = [ 0 norm_r0 ];                              % B = [ beta_0 beta_1 ]
G = [ [1 0] ; [1 0] ];                          % G = [ givens_rotation_0; givens_rotation_1 ]
U = [ 1 1 ];                                    % U = [ u_-1(-1) u_-2(-2) ]
W = [ zeros(dim,1) zeros(dim,1) ];              % W = [ w'_0 w'_-1 ] ( NOTE: w'_j = u_j(j)*w_j )
clear r;

%% MRS3 iteration loop
while (res_norm/norm_r0 > tol && j<maxiter)

   j = j+1;                                     % increase iteration counter
   t = it; it = io; io = t;                     % swap index_this & index_other

   % SSS Lanczos
   Q(:,it) = -Q(:,it)/B(:,it);                  % q_j = -p_j/b_j
   Q(:,io) = S*Q(:,it) - B(:,it)*Q(:,io);       % p_{j+1} = S*q_j - beta_j*q_{j-1}
   B(:,io) = norm(Q(:,io),2);                   % beta_{j+1} = ||p_{j+1}||

   % Build the parts of u and v that are needed
   u = [ 0 ; B(:,it) ; alpha ; -B(:,io) ];      % u = [ 0; beta_j; alpha; beta_{j+1} ]
   v = [ eps ; 0 ];                             % v = [ v_{j-1}(j); 0 ]

   % Apply Givens rotation j-2 on u
   y = u(1); z = u(2);
   u(1) = G(it,1)*y + G(it,2)*z;
   u(2) = G(it,1)*z - G(it,2)*y;

   % Apply Givens rotation j-1 on u
   y = u(2); z = u(3);
   u(2) = G(io,1)*y + G(io,2)*z;
   u(3) = G(io,1)*z - G(io,2)*y;

   % Apply Givens rotation j on u and v
   if (abs(u(4)) >= abs(u(3)))
      t = u(3)/u(4);
      G(it,2) = 1/(sqrt(1+t^2));
      G(it,1) = G(it,2)*t;
   else
      t = u(4)/u(3);
      G(it,1) = 1/(sqrt(1+t^2));
      G(it,2) = G(it,1)*t;
   end
   y = u(3); z = u(4);
   u(3) = G(it,1)*y + G(it,2)*z;
   u(4) = G(it,1)*z - G(it,2)*y;
   y = v(1); z = v(2);
   v(1) = G(it,1)*y + G(it,2)*z;
   v(2) = G(it,1)*z - G(it,2)*y;

   % Update solution
   W(:,it) = Q(:,it) - (u(1)/U(:,it))*W(:,it);  % w'_j = q_j - u_j(j-2)/u_{j-2}(j-2)*w'_{j-2}
   U(:,it) = u(3);                              % update u_{j-2}(j-2) into u_j(j)
   x = x + (v(1)/U(:,it))*W(:,it);              % x = x + v_j(j)/u_j(j)*w'_j
   eps = v(2);                                  % eps = v_j(j+1)
   res_norm = abs(eps);                         % res_norm = abs(eps) = ||r_j||
   res_hist(j+1) = res_norm;

end

%% End
