function [A, b] = testsystem(alpha, ratio)
%TESTSYSTEM Builds a shifted skew-symmetric matrix system A = alpha*I + S,
%with S skew-symmetric, for testing purposes.
%
%  Authors     :  Reijer Idema (http://www.reijeridema.nl)
%              :  Kees Vuik (http://ta.twi.tudelft.nl/users/vuik)
%  Last edited :  15 September 2008
%
%  Parameters
%     alpha    :  diagonal shift of the coefficient matrix
%     ratio    :  multiplier for extra subdiagonal and superdiagonal values
%
%  Output
%     A        :  shifted skew-symmetric coefficient matrix
%     b        :  random right hand side vector with norm 1.

%  This test system was proposed by Kees Vuik, and was used to test
%  solvers in our research on Krylov methods for SSS systems.

%% Initialize variables
n1 = 20;
n2 = 20;

h1 = 1/n1;
h2 = 1/n2;

dim = n1*n2;
M = sparse(dim,dim);

%% Build tridiagonal blocks
v1 = 1/(2*h1);
for i=1:dim - 1
   M(i,i+1) = v1;
	M(i+1,i) = -v1;
end
for i = 1:n2-1
   M(i*n1,i*n1+1) = 0;
   M(i*n1+1,i*n1) = 0;
end

%% Build extra subdiagonal and superdiagonal
v2 = ratio/(2*h2);
for i=1:dim - n1
   M(i,i+n1) = v2;
   M(i+n1,i) = -v2;
end

%% Add shift
if alpha~=0
   A = M + alpha*eye(dim);
end

%% Build right hand side vector
rand('state',0);
b = rand(dim,1);
b = b/norm(b);

%% End
